package cn.wise.sc.consume.business.controller;

import cn.wise.sc.consume.business.util.UserUtil;
import cn.wise.sc.consume.business.util.dfs.FastDFSUtils;
import cn.wise.sc.consume.business.util.dfs.FileTypeUtil;
import cn.wise.sc.consume.business.wrapper.WrapMapper;
import cn.wise.sc.consume.business.wrapper.Wrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 附件管理Controller
 *  @author ztw
 */
@Api(tags = "附件管理")
@RestController
@RequestMapping("/enclosure")
public class EntityEnclosureController  {

	private static final Logger logger = LoggerFactory.getLogger("adminLogger");

	@Autowired
	protected HttpServletResponse response;
	@Autowired
	protected HttpServletRequest request;



	@ApiOperation(value = "单个文件上传")
	@PostMapping("/fileUpLoad")
	public Wrapper fileUpLoad(MultipartFile file){
		Map<String,Object> map = new HashMap<>();
		String filePath = null;
		String fileName = null;
		String extName = null;
		try {
			Long userId = UserUtil.getLoginUser().getId();
			if(userId != null){
				filePath = FastDFSUtils.uploadPic(file.getBytes(),file.getOriginalFilename(),file.getSize());
				fileName = file.getOriginalFilename().substring(0,file.getOriginalFilename().lastIndexOf("."));
				extName = file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf("."));
			}else {
				return WrapMapper.wrap(400,"该账号已在其他地方登陆");
			}
			String picUrl = filePath;
			map.put("fileUrl",picUrl);
			map.put("fileName",fileName);
			map.put("extName",extName);
			return WrapMapper.wrap(200,"成功", map);
		}catch (Exception e){
			logger.error(e.toString());
		}
		return WrapMapper.error("失败");
	}

	@ApiOperation(value = "单个图片上传")
	@PostMapping("/imgUpLoad")
	public Wrapper imgUpLoad(MultipartFile file){
		Map<String,Object> map = new HashMap<>();
		String filePath = null;
		String fileName = null;
		String extName = null;
		try {

			Long userId = UserUtil.getLoginUser().getId();
			if(userId != null){
				if(!FileTypeUtil.isImageByExtension(file.getOriginalFilename())){
					return WrapMapper.wrap(400,"请上传有效图片");
				}
				filePath = FastDFSUtils.uploadPic(file.getBytes(),file.getOriginalFilename(),file.getSize());
				fileName = file.getOriginalFilename().substring(0,file.getOriginalFilename().lastIndexOf("."));
				extName = file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf("."));
			}else {
				return WrapMapper.wrap(400,"该账号已在其他地方登陆");
			}
			String picUrl =  filePath;
			map.put("fileUrl",picUrl);
			map.put("fileName",fileName);
			map.put("extName",extName);
			return WrapMapper.wrap(200,"成功", map);
		}catch (Exception e){
			logger.error(e.toString());
		}
		return WrapMapper.error("失败");
	}


    @ApiOperation(value = "多个文件上传")
    @PostMapping("/filesUpLoad")
    public Wrapper filesUpLoad(HttpServletRequest request){
        Map<String,Object> map = new HashMap<>();
        List<MultipartFile> files = null;
        String filePath = null;
	    String fileName = null;
	    String extName = null;
        List<Map<String,Object>> filePathList = null;
        try {
            filePathList = new ArrayList<>();
            files = ((MultipartHttpServletRequest)request).getFiles("files");
	        Long userId = UserUtil.getLoginUser().getId();
            if(userId != null){
                if(files != null && files.size() > 0) {
                    for (MultipartFile file : files) {
	                    Map<String,Object> mapSub = new HashMap<>();
                        filePath = FastDFSUtils.uploadPic(file.getBytes(), file.getOriginalFilename(), file.getSize());
	                    fileName = file.getOriginalFilename().substring(0,file.getOriginalFilename().lastIndexOf("."));
	                    extName = file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf("."));
	                    String picUrl =  filePath;
	                    mapSub.put("fileUrl",picUrl);
	                    mapSub.put("fileName",fileName);
	                    mapSub.put("extName",extName);
                        filePathList.add(mapSub);
                    }
                }else {
	                return WrapMapper.wrap(400,"没有文件");
                }
            }else {
	            return WrapMapper.wrap(400,"该账号已在其他地方登陆");
            }
            map.put("status",200);
            map.put("message","上传成功");
            map.put("filePathList",filePathList);
	        return WrapMapper.wrap(200,"成功", filePathList);
        }catch (Exception e){
            logger.error(e.toString());
        }
	    return WrapMapper.error("失败");
    }

    @ApiOperation(value = "单个文件下载")
    @PostMapping("/fileDownload")
    public Wrapper fileDownload(String fileUrl,String fileName,String extName){
        OutputStream out = null;
        try {
            response.setHeader("Content-Disposition", "attachment;filename="+fileName+extName);
            response.setContentType("application/force-download");
            out = response.getOutputStream();
	        Long userId = UserUtil.getLoginUser().getId();
            if(userId != null){
                fileUrl = fileUrl.substring(fileUrl.indexOf("g"));
                byte[] by = FastDFSUtils.fileDownload(fileUrl);
                out.write(by);
                out.close();
            }else {
	            return WrapMapper.wrap(400,"该账号已在其他地方登陆");
            }
	        return WrapMapper.wrap(200,"成功");
        }catch (Exception e){
            logger.error(e.toString());
        }
	    return WrapMapper.error("失败");
    }

    @ApiOperation(value = "app扫描二维码下载")
	@GetMapping("/appDownload")
    public Wrapper appDownload(String fileUrl){
        Map<String,Object> map = new HashMap<>();
        OutputStream out = null;
        try {
            response.setHeader("Content-Disposition", "attachment;filename=app-release.apk");
            response.setContentType("application/force-download");
            out = response.getOutputStream();
            fileUrl = fileUrl.substring(fileUrl.indexOf("g"));
            byte[] by = FastDFSUtils.fileDownload(fileUrl);
            out.write(by);
            out.close();
	        return WrapMapper.wrap(200,"成功");

        }catch (Exception e){
            logger.error(e.toString());
        }
	    return WrapMapper.error("失败");
    }



    @ApiOperation(value = "删除相关文件")
    @PostMapping("/fileDeleted")
    public Wrapper fileDeleted(String[] filePaths){
        Map<String,Object> map = new HashMap<>();
        String filePath = null;
        boolean rel = false;
        try {
	        Long userId = UserUtil.getLoginUser().getId();
            if(userId != null){
                for(String url : filePaths){
                    /*filePath = url.substring(0,url.indexOf("g"));*/
                    rel = FastDFSUtils.deletePic(url);
                    if(!rel){
	                    return WrapMapper.wrap(400,"删除失败");
                    }
                }
            }else {
	            return WrapMapper.wrap(400,"该账号已在其他地方登陆");
            }
	        return WrapMapper.wrap(200,"成功");
        }catch (Exception e){
            logger.error(e.toString());
        }
	    return WrapMapper.error("失败");
    }


}
